︠f0407ff8-43f4-486c-8fdd-2edd46ed87ads︠
# Kaip papildyti vektorių sistemą iki erdvės bazės?

def papildom_poerdviai(W, V):
    """
    Gražina tiesinės erdvės V poerdvio W papildinio iki V bazę. 
    Kitaip sakant, randa tokį poerdvį U, kad V yra W ir U tiesioginė 
    suma ir gražina jo bazę.
    """
    Q, pi, lift = V.quotient_abstract(W)
    Basis = [lift(v) for v in Q.basis()]
    return Basis

# pi   yra projekcijos atvaizdis iš erdvės V į faktorerdvę V/W: pi(v) = v + W.
# lift yra "atvirkštinis" atvaizdis iš faktorerdvės Q = V/W  į  erdvę V, tenkinantis 
# lygybę pi(lift(u)) = u  su visais u iš Q.


def papildom(vec, V):
    """
    Tiesinės erdvės V vektorių šeimą vec papildo vektoriais v1,..., vn iki 
    erdvės V bazės ir gražina papildytus vektorius v1,..., vn.
    """
    return papildom_poerdviai(V.span(vec), V)

︡303d0427-4427-4410-a1c0-02b0fc1c7db9︡{"done":true}
︠59876658-4e86-4924-b8cb-1a7385dfc614s︠
# Pavyzdžiui,
vec = [vector([1,0,0]),vector([1,1,0])]
papildom(vec, QQ^3)

︡c3ff79c6-9ade-4f71-83d9-d4685c47a29b︡{"stdout":"[(0, 0, 1)]\n"}︡{"done":true}
︠11bb3442-9373-4eca-b07c-cf87c7e2f668︠

︡f12abe99-5500-4872-befa-7503911124e0︡
︠eb616ba1-fa26-41ef-a4ca-ca91e8784179s︠
# 1 pavyzdys. Rasime matricos

A = matrix(QQ, 4, 4, [[5, 1, -1, -1],
                      [1, 5, -1, -1],
                      [1, 1, 3, -1],
                      [1,1,-1,3]])
show(A)
︡181933e2-ac66-4e3f-a534-38b4448c5376︡{"html":"<div align='center'>$\\displaystyle \\left(\\begin{array}{rrrr}\n5 &amp; 1 &amp; -1 &amp; -1 \\\\\n1 &amp; 5 &amp; -1 &amp; -1 \\\\\n1 &amp; 1 &amp; 3 &amp; -1 \\\\\n1 &amp; 1 &amp; -1 &amp; 3\n\\end{array}\\right)$</div>"}︡{"done":true}
︠ab4244ce-99b2-46fd-80b5-644d1ecb786c︠
# Žodano formą ir atitinkamą bazę.
︡affd63e5-63cc-4149-9317-875eac9ae867︡
︠790645b1-219a-4a90-8287-9c6b11c9e69c︠

%md 
︡725b86d5-b909-4305-b8d7-d65d6652aacf︡{"md":""}︡
︠93cdbdc9-9613-4440-8f13-834b69af7c3es︠
# Randame matricos charakteristinį polinomą:

p = A.charpoly(var = 't')
show(p)
show(p.factor())
︡e63e6afd-d123-477f-b118-edd7988b4520︡{"html":"<div align='center'>$\\displaystyle t^{4} - 16 t^{3} + 96 t^{2} - 256 t + 256$</div>"}︡{"html":"<div align='center'>$\\displaystyle (t - 4)^{4}$</div>"}︡{"done":true}
︠026db0e1-9997-42f6-ad96-bc14a86b8d66s︠
# Nagrinėjame matricą B := A-4*E. Ieškome mažiausoi p: rg(B^p)=4-4=0:
B = A - 4*matrix.identity(4)
show(B)
︡7c2b7b71-e627-45f3-98fc-c357af83bab8︡{"html":"<div align='center'>$\\displaystyle \\left(\\begin{array}{rrrr}\n1 &amp; 1 &amp; -1 &amp; -1 \\\\\n1 &amp; 1 &amp; -1 &amp; -1 \\\\\n1 &amp; 1 &amp; -1 &amp; -1 \\\\\n1 &amp; 1 &amp; -1 &amp; -1\n\\end{array}\\right)$</div>"}︡{"done":true}
︠0679cb04-262e-452a-9504-e491ada33f67s︠
# Matricos B laipsnių rangai:
B.rank()
(B**2).rank()
︡0f54834f-db92-4094-8852-96fcd514da78︡{"stdout":"1\n"}︡{"stdout":"0\n"}︡{"done":true}
︠91e33067-49ed-46c0-a6d3-eb8c26e783b3s︠
# Taigi, p=2. Matricos B branduolys ker B ir jo dimensija:
L1 = B.left_kernel()
L1.dimension()
︡ab5a2df1-aed6-45fb-9a0f-6e19ac312843︡{"stdout":"3\n"}︡{"done":true}
︠561a0d81-5b90-4a8e-a9d8-548373343eads︠
# Matricos B^2 branduolys ker B^2 ir jo dimensija:
L2 = (B**2).left_kernel()
L2.dimension()
︡88918b23-5699-40de-844b-37bacd50ab8b︡{"stdout":"4\n"}︡{"done":true}
︠171492ee-30e9-4a28-948e-6b95af7b2f2bs︠
# Poerdvio L1 bazę pildant iki erdvės L2 bazės, reikia pridėti 1 vektorių, kurį
# atitiks lygiai vianas 2-os eilės Žordano langelis. 
# Rasime šį vektorių:

v11 = papildom(L1.basis(), L2)[0]
v11
︡bbbf4689-bd61-47a0-8c55-230a8037d916︡{"stdout":"(1, 0, 0, 0)\n"}︡{"done":true}
︠37f72aa0-d152-4557-b9a3-8bccf955e21cs︠
# Suskaičiuojame dar vieną vektorių:

v11B = v11*B  # priklauso L1
v11B
︡dc58feb2-308b-4836-b5aa-30886d6269d6︡{"stdout":"(1, 1, -1, -1)\n"}︡{"done":true}
︠42d581cb-3a75-4a1e-92a1-6c606b8d6ba8s︠
# Kadangi poerdvio L1 dimensija lygi 3 ir v11B priklauso L1, tai vektorių v11B reikia papildyti dviem vektoriais v21,v22 iki L1 bazės:
L = papildom([v11B], L1)
v21 = L[0]
v22 = L[1]
v21
v22
︡ec7506ca-93c2-45bd-9ebd-2d0b44b93e0e︡{"stdout":"(1, 0, 0, -1)\n"}︡{"stdout":"(0, 1, 0, -1)\n"}︡{"done":true}
︠ee05924e-c11d-48e6-a106-c0569d3049a0s︠
# Taigi radome Žordano bazę v11,v11B,v21,v22. 
# Perėjimo matrica (iš standartinės bazės į Žordano bazę):

T = matrix(QQ,4,4,[v11,v11B,v21,v22])
show(T)
︡875ac86f-7855-4711-b396-eba05f30094f︡{"html":"<div align='center'>$\\displaystyle \\left(\\begin{array}{rrrr}\n1 &amp; 0 &amp; 0 &amp; 0 \\\\\n1 &amp; 1 &amp; -1 &amp; -1 \\\\\n1 &amp; 0 &amp; 0 &amp; -1 \\\\\n0 &amp; 1 &amp; 0 &amp; -1\n\\end{array}\\right)$</div>"}︡{"done":true}
︠71220edc-2b06-4d68-b5b2-9ff06067ba9as︠
# Pradinės matricos Žordano matrica:

show(T*A*(T^(-1)))
︡5f9c0342-468c-46bd-87b8-dd862d04c4b0︡{"html":"<div align='center'>$\\displaystyle \\left(\\begin{array}{rrrr}\n4 &amp; 1 &amp; 0 &amp; 0 \\\\\n0 &amp; 4 &amp; 0 &amp; 0 \\\\\n0 &amp; 0 &amp; 4 &amp; 0 \\\\\n0 &amp; 0 &amp; 0 &amp; 4\n\\end{array}\\right)$</div>"}︡{"done":true}
︠26f5b6ef-1b88-46b0-8fdf-d6438042c2e4︠









